package ru.dbotthepony.kommons.arrays

/**
 * Operations over 2D arrays are done in-place, unless states otherwise
 */
abstract class Array2D {
	abstract val rows: Int
	abstract val columns: Int

	val size: Int get() = rows * columns
	fun size() = size

	val isEmpty: Boolean get() = size <= 0
	val isNotEmpty: Boolean get() = size > 0

	val isSquareShaped: Boolean get() = rows == columns

	inline val columnIndices get() = 0 until columns
	inline val rowIndices get() = 0 until rows
	inline val dimensionsString get() = "$columns x $rows"

	fun sizeEquals(other: Array2D): Boolean {
		return rows == other.rows && columns == other.columns
	}

	fun requireSizeEquals(other: Array2D) {
		if (!sizeEquals(other)) {
			throw IllegalArgumentException("Other matrix (${other.columns}x${other.rows}) is not the same dimensions as this matrix (${columns}x${rows})")
		}
	}

	inline fun requireSizeEquals(other: Array2D, lambda: () -> Any) {
		if (!sizeEquals(other)) {
			throw IllegalArgumentException(lambda.invoke().toString())
		}
	}

	fun checkSizeEquals(other: Array2D) {
		if (!sizeEquals(other)) {
			throw IllegalStateException("Other matrix (${other.columns}x${other.rows}) is not the same dimensions as this matrix (${columns}x${rows})")
		}
	}

	inline fun checkSizeEquals(other: Array2D, lambda: () -> Any) {
		if (!sizeEquals(other)) {
			throw IllegalStateException(lambda.invoke().toString())
		}
	}

	fun rowsEquals(other: Array2D): Boolean {
		return rows == other.rows
	}

	fun requireRowsEquals(other: Array2D) {
		if (!rowsEquals(other)) {
			throw IllegalArgumentException("Other matrix (${other.rows}) has different amount of rows compared to this matrix (${rows})")
		}
	}

	inline fun requireRowsEquals(other: Array2D, lambda: () -> Any) {
		if (!rowsEquals(other)) {
			throw IllegalArgumentException(lambda.invoke().toString())
		}
	}

	fun checkRowsEquals(other: Array2D) {
		if (!rowsEquals(other)) {
			throw IllegalStateException("Other matrix (${other.rows}) has different amount of rows compared to this matrix (${rows})")
		}
	}

	inline fun checkRowsEquals(other: Array2D, lambda: () -> Any) {
		if (!rowsEquals(other)) {
			throw IllegalStateException(lambda.invoke().toString())
		}
	}

	fun columnsEquals(other: Array2D): Boolean {
		return columns == other.columns
	}

	fun requireColumnsEquals(other: Array2D) {
		if (!rowsEquals(other)) {
			throw IllegalArgumentException("Other matrix (${other.columns}) has different amount of columns compared to this matrix (${columns})")
		}
	}

	inline fun requireColumnsEquals(other: Array2D, lambda: () -> Any) {
		if (!rowsEquals(other)) {
			throw IllegalArgumentException(lambda.invoke().toString())
		}
	}

	fun checkColumnsEquals(other: Array2D) {
		if (!rowsEquals(other)) {
			throw IllegalStateException("Other matrix (${other.columns}) has different amount of columns compared to this matrix (${columns})")
		}
	}

	inline fun checkColumnsEquals(other: Array2D, lambda: () -> Any) {
		if (!rowsEquals(other)) {
			throw IllegalStateException(lambda.invoke().toString())
		}
	}

	inline fun requireIsSquare(lambda: () -> Any) {
		if (!isSquareShaped) {
			throw IllegalArgumentException(lambda.invoke().toString())
		}
	}

	fun requireIsSquare() {
		if (!isSquareShaped) {
			throw IllegalArgumentException("Matrix is not square matrix: $columns x $rows")
		}
	}

	inline fun checkIsSquare(lambda: () -> Any) {
		if (!isSquareShaped) {
			throw IllegalStateException(lambda.invoke().toString())
		}
	}

	fun checkIsSquare() {
		if (!isSquareShaped) {
			throw IllegalStateException("Matrix is not square matrix: $columns x $rows")
		}
	}
}
