package ru.dbotthepony.kommons.io

import ru.dbotthepony.kommons.util.KOptional
import java.io.Closeable
import java.io.File

/**
 * For actual implementation, see [BTreeDB6].
 */
abstract class BTreeDB<K, V> : Closeable {
	abstract val file: File
	abstract val blockSize: Int

	abstract operator fun contains(key: K): Boolean

	/**
	 * Reads data at specified [key]
	 */
	abstract fun read(key: K): KOptional<V>
	abstract fun findAllKeys(): List<K>

	abstract fun write(key: K, value: V)
}

abstract class ByteDataBTreeDB<K> : BTreeDB<K, ByteArray>() {
	abstract fun write(key: K, value: ByteArray, offset: Int, length: Int)

	final override fun write(key: K, value: ByteArray) {
		return write(key, value, 0, value.size)
	}
}
